# -*- coding: utf-8 -*-
import copy
from abjad.tools import mathtools
from abjad.tools.pitchtools.Set import Set
from abjad.tools.topleveltools import new


class PitchClassSet(Set):
    '''Pitch-class set.

    ..  container:: example

        Initializes numbered pitch-class set:

        ::

            >>> numbered_pitch_class_set = PitchClassSet(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     item_class=NumberedPitchClass,
            ...     )
            >>> numbered_pitch_class_set
            PitchClassSet([6, 7, 10, 10.5])

    ..  container:: example

        Initializes named pitch-class set:

        ::

            >>> named_pitch_class_set = PitchClassSet(
            ...     items=['c', 'ef', 'bqs,', 'd'],
            ...     item_class=NamedPitchClass,
            ...     )
            >>> named_pitch_class_set
            PitchClassSet(['c', 'd', 'ef', 'bqs'])

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### SPECIAL METHODS ###

    def __hash__(self):
        r'''Hashes pitch-class set.

        Returns integer.
        '''
        return hash(repr(self))

    def __illustrate__(self):
        r'''Illustrates pitch-class set.

        ..  container:: example

            Illustrates numbered segment:

            ::

                >>> set_ = PitchClassSet([-2, -1.5, 6, 7, -1.5, 7])
                >>> show(set_) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = set_.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    <fs' g' bf' bqf'>1
                }

        ..  container:: example

            Illustrates named set:

            ::

                >>> items = ['c', 'ef', 'bqs,', 'd']
                >>> set_ = PitchClassSet(
                ...     items=items,
                ...     item_class=NamedPitchClass,
                ...     )
                >>> show(set_) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = set_.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    <c' d' ef' bqs'>1
                }

        ..  container:: example

            Returns LilyPond file:

            ::

                >>> prototype = lilypondfiletools.LilyPondFile
                >>> isinstance(set_.__illustrate__(), prototype)
                True

        '''
        import abjad
        chord = abjad.Chord(self, abjad.Duration(1))
        voice = abjad.Voice([chord])
        staff = abjad.Staff([voice])
        score = abjad.Score([staff])
        lilypond_file = abjad.LilyPondFile.new(score)
        lilypond_file.header_block.tagline = False
        return lilypond_file

    def __str__(self):
        r'''Gets string representation of pitch-class set.

        ..  container:: example

            Gets string of set sorted at initialization:

            ::

                >>> pc_set = PitchClassSet([6, 7, 10, 10.5])
                >>> str(pc_set)
                'PC{6, 7, 10, 10.5}'

        ..  container:: example

            Gets string of set not sorted at initialization:

            ::

                >>> pc_set = PitchClassSet([10.5, 10, 7, 6])
                >>> str(pc_set)
                'PC{6, 7, 10, 10.5}'

        Returns string.
        '''
        import abjad
        items = [str(_) for _ in sorted(self)]
        separator = ' '
        if self.item_class is abjad.NumberedPitchClass:
            separator = ', '
        return 'PC{{{}}}'.format(separator.join(items))

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NamedPitchClass

    @property
    def _numbered_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitchClass

    @property
    def _parent_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.PitchClass

    ### PRIVATE METHODS ###

    @staticmethod
    def _get_most_compact_ordering(candidates):
        from abjad.tools import pitchtools
        widths = []
        for candidate in candidates:
            if candidate[0] < candidate[-1]:
                width = abs(candidate[-1] - candidate[0])
            else:
                width = abs(candidate[-1] + 12 - candidate[0])
            widths.append(width)
        minimum_width = min(widths)
        candidates_ = []
        for candidate, width in zip(candidates, widths):
            if width == minimum_width:
                candidates_.append(candidate)
        candidates = candidates_
        assert 1 <= len(candidates)
        if len(candidates) == 1:
            segment = candidates[0]
            segment = pitchtools.PitchClassSegment(
                items=segment,
                item_class=pitchtools.NumberedPitchClass,
                )
            return segment
        for i in range(len(candidates[0]) - 1):
            widths = []
            for candidate in candidates:
                if candidate[0] < candidate[i+1]:
                    width = abs(candidate[i+1] - candidate[0])
                else:
                    width = abs(candidate[i+1] + 12 - candidate[0])
                widths.append(width)
            minimum_width = min(widths)
            candidates_ = []
            for candidate, width in zip(candidates, widths):
                if width == minimum_width:
                    candidates_.append(candidate)
            candidates = candidates_
            if len(candidates) == 1:
                segment = candidates[0]
                segment = pitchtools.PitchClassSegment(
                    items=segment,
                    item_class=pitchtools.NumberedPitchClass,
                    )
                return segment
        candidates.sort(key=lambda x: x[0])
        segment = candidates[0]
        segment = pitchtools.PitchClassSegment(
            items=segment,
            item_class=pitchtools.NumberedPitchClass,
            )
        return segment

    def _sort_self(self):
        from abjad.tools import pitchtools
        def helper(x, y):
            return cmp(
                pitchtools.NamedPitch(pitchtools.NamedPitchClass(x), 0),
                pitchtools.NamedPitch(pitchtools.NamedPitchClass(y), 0)
                )
        result = list(self)
        result.sort(helper)
        return result

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(
        class_,
        selection,
        item_class=None,
        ):
        r'''Makes pitch-class set from `selection`.

        ::

            >>> staff_1 = Staff("c'4 <d' fs' a'>4 b2")
            >>> staff_2 = Staff("c4. r8 g2")
            >>> selection = select((staff_1, staff_2))
            >>> PitchClassSet.from_selection(selection)
            PitchClassSet(['c', 'd', 'fs', 'g', 'a', 'b'])

        Returns pitch-class set.
        '''
        from abjad.tools import pitchtools
        pitch_segment = pitchtools.PitchSegment.from_selection(selection)
        return class_(
            items=pitch_segment,
            item_class=item_class,
            )

    def get_normal_order(self):
        r'''Gets normal order.

        ..  container:: example

            Gets normal order of empty pitch-class set:

            ::

                >>> pc_set = PitchClassSet()
                >>> pc_set.get_normal_order()
                PitchClassSegment([])

        ..  container:: example

            Gets normal order:

            ::

                >>> pc_set = PitchClassSet([0, 1, 10, 11])
                >>> pc_set.get_normal_order()
                PitchClassSegment([10, 11, 0, 1])

        ..  container:: example

            Gets normal order:

            ::

                >>> pc_set = PitchClassSet([2, 8, 9])
                >>> pc_set.get_normal_order()
                PitchClassSegment([8, 9, 2])

        ..  container:: example

            Gets normal order of pitch-class set with degree of symmetry equal
            to 2:

            ::

                >>> pc_set = PitchClassSet([1, 2, 7, 8])
                >>> pc_set.get_normal_order()
                PitchClassSegment([1, 2, 7, 8])

        ..  container:: example

            Gets normal order of pitch-class set with degree of symmetry equal
            to 4:

            ::

                >>> pc_set = PitchClassSet([0, 3, 6, 9])
                >>> pc_set.get_normal_order()
                PitchClassSegment([0, 3, 6, 9])

        Returns pitch-class segment.
        '''
        from abjad.tools import pitchtools
        from abjad.tools import sequencetools
        if not len(self):
            return pitchtools.PitchClassSegment(
                items=None,
                item_class=pitchtools.NumberedPitchClass,
                )
        pitch_classes = list(self)
        pitch_classes.sort()
        candidates = []
        for i in range(self.cardinality):
            candidate = [pitchtools.NumberedPitch(_) for _ in pitch_classes]
            candidate = sequencetools.Sequence(candidate).rotate(n=-i)
            candidates.append(candidate)
        return self._get_most_compact_ordering(candidates)

    def get_prime_form(self, transposition_only=False):
        r'''Gets prime form.

        ..  container:: example

            Gets prime form of empty pitch-class set:

            ::

                >>> pc_set = PitchClassSet()
                >>> pc_set.get_prime_form()
                PitchClassSet([])

            ::

                >>> pc_set = PitchClassSet()
                >>> pc_set.get_prime_form(transposition_only=True)
                PitchClassSet([])

        ..  container:: example

            Gets prime form:

            ::

                >>> pc_set = PitchClassSet([0, 1, 10, 11])
                >>> pc_set.get_prime_form()
                PitchClassSet([0, 1, 2, 3])

            ::

                >>> pc_set = PitchClassSet([0, 1, 10, 11])
                >>> pc_set.get_prime_form(transposition_only=True)
                PitchClassSet([0, 1, 2, 3])

        ..  container:: example

            Gets prime form:

            ::

                >>> pc_set = PitchClassSet([2, 8, 9])
                >>> pc_set.get_prime_form()
                PitchClassSet([0, 1, 6])

            ::

                >>> pc_set = PitchClassSet([2, 8, 9])
                >>> pc_set.get_prime_form(transposition_only=True)
                PitchClassSet([0, 1, 6])

        ..  container:: example

            Gets prime form of pitch-class set with degree of symmetry equal to
            2:

            ::

                >>> pc_set = PitchClassSet([1, 2, 7, 8])
                >>> pc_set.get_prime_form()
                PitchClassSet([0, 1, 6, 7])

            ::

                >>> pc_set = PitchClassSet([1, 2, 7, 8])
                >>> pc_set.get_prime_form(transposition_only=True)
                PitchClassSet([0, 1, 6, 7])

        ..  container:: example

            Gets prime form of pitch-class set with degree of symmetry equal to
            4:

            ::

                >>> pc_set = PitchClassSet([0, 3, 6, 9])
                >>> pc_set.get_prime_form()
                PitchClassSet([0, 3, 6, 9])

            ::

                >>> pc_set = PitchClassSet([0, 3, 6, 9])
                >>> pc_set.get_prime_form(transposition_only=True)
                PitchClassSet([0, 3, 6, 9])

        ..  container:: example

            Gets prime form of pitch-class that is not inversion-equivalent:

            ::

                >>> pc_set = PitchClassSet([0, 4, 6, 7])
                >>> pc_set.get_prime_form()
                PitchClassSet([0, 1, 3, 7])

            ::

                >>> pc_set = PitchClassSet([0, 4, 6, 7])
                >>> pc_set.get_prime_form(transposition_only=True)
                PitchClassSet([0, 4, 6, 7])

        Returns new pitch-class set.
        '''
        from abjad.tools import pitchtools
        if not len(self):
            return copy.copy(self)
        normal_orders = [self.get_normal_order()]
        if not transposition_only:
            inversion = self.invert()
            normal_order = inversion.get_normal_order()
            normal_orders.append(normal_order)
        normal_order = self._get_most_compact_ordering(normal_orders)
        pcs = [int(_) for _ in normal_order]
        first_pc = pcs[0]
        pcs = [pc - first_pc for pc in pcs]
        prime_form = type(self)(
            items=pcs,
            item_class=pitchtools.NumberedPitchClass,
            )
        return prime_form

    def invert(self, axis=None):
        r'''Inverts pitch-class set.

        ::

            >>> PitchClassSet(
            ...     [-2, -1.5, 6, 7, -1.5, 7],
            ...     ).invert()
            PitchClassSet([1.5, 2, 5, 6])

        Returns numbered pitch-class set.
        '''
        return type(self)([pc.invert(axis=axis) for pc in self])

    def is_transposed_subset(self, pcset):
        r'''Is true when pitch-class set is transposed subset of `pcset`.
        Otherwise false:

        ::

            >>> pitch_class_set_1 = PitchClassSet(
            ...     [-2, -1.5, 6, 7, -1.5, 7],
            ...     )
            >>> pitch_class_set_2 = PitchClassSet(
            ...     [-2, -1.5, 6, 7, -1.5, 7, 7.5, 8],
            ...     )

        ::

            >>> pitch_class_set_1.is_transposed_subset(pitch_class_set_2)
            True

        Returns true or false.
        '''
        for n in range(12):
            if self.transpose(n).issubset(pcset):
                return True
        return False

    def is_transposed_superset(self, pcset):
        r'''Is true when pitch-class set is transposed superset of `pcset`.
        Otherwise false:

        ::

            >>> pitch_class_set_1 = PitchClassSet(
            ...     [-2, -1.5, 6, 7, -1.5, 7],
            ...     )
            >>> pitch_class_set_2 = PitchClassSet(
            ...     [-2, -1.5, 6, 7, -1.5, 7, 7.5, 8],
            ...     )

        ::

            >>> pitch_class_set_2.is_transposed_superset(pitch_class_set_1)
            True

        Returns true or false.
        '''
        for n in range(12):
            if self.transpose(n).issuperset(pcset):
                return True
        return False

    def multiply(self, n):
        r'''Multiplies pitch-class set by `n`.

        ::

            >>> PitchClassSet(
            ...     [-2, -1.5, 6, 7, -1.5, 7],
            ...     ).multiply(5)
            PitchClassSet([2, 4.5, 6, 11])

        Returns new pitch-class set.
        '''
        items = (pitch_class.multiply(n) for pitch_class in self)
        return new(self, items=items)

    def order_by(self, pitch_class_segment):
        r'''Orders pitch-class set by `pitch_class_segment`.

        Returns pitch-class segment.
        '''
        from abjad.tools import pitchtools
        from abjad.tools import sequencetools
        if not len(self) == len(pitch_class_segment):
            message = 'set and segment must be on equal length.'
            raise ValueError(message)
        enumeration = sequencetools.Enumeration(self)
        for pitch_classes in enumeration.yield_permutations():
            candidate_pitch_class_segment = \
                pitchtools.PitchClassSegment(pitch_classes)
            if candidate_pitch_class_segment._is_equivalent_under_transposition(
                pitch_class_segment):
                return candidate_pitch_class_segment
        message = 'named pitch-class set {} can not order by '
        message += 'named pitch-class segment {}.'
        message = message.format(self, pitch_class_segment)
        raise ValueError(message)

    def transpose(self, n=0):
        r'''Transposes all pitch-classes in pitch-class set by index `n`.

        Returns new pitch-class set.
        '''
        items = (pitch_class + n for pitch_class in self)
        return new(self, items=items)

    @staticmethod
    def yield_all_pitch_class_sets():
        '''Yields all pitch-class sets.

        ..  container:: example

            Yields all pitch-class sets:

            ::


                >>> class_ = PitchClassSet
                >>> pcsets = list(class_.yield_all_pitch_class_sets())
                >>> len(pcsets)
                4096

            ::

                >>> for pcset in pcsets[:20]:
                ...   pcset
                PitchClassSet([])
                PitchClassSet([0])
                PitchClassSet([1])
                PitchClassSet([0, 1])
                PitchClassSet([2])
                PitchClassSet([0, 2])
                PitchClassSet([1, 2])
                PitchClassSet([0, 1, 2])
                PitchClassSet([3])
                PitchClassSet([0, 3])
                PitchClassSet([1, 3])
                PitchClassSet([0, 1, 3])
                PitchClassSet([2, 3])
                PitchClassSet([0, 2, 3])
                PitchClassSet([1, 2, 3])
                PitchClassSet([0, 1, 2, 3])
                PitchClassSet([4])
                PitchClassSet([0, 4])
                PitchClassSet([1, 4])
                PitchClassSet([0, 1, 4])

        There are 4096 pitch-class sets.

        This is ``U*`` in [Morris 1987].

        Returns generator.
        '''
        from abjad.tools import pitchtools
        def _helper(binary_string):
            result = zip(binary_string, range(len(binary_string)))
            result = [string[1] for string in result if string[0] == '1']
            return result
        for i in range(4096):
            string = mathtools.integer_to_binary_string(i).zfill(12)
            subset = ''.join(list(reversed(string)))
            subset = _helper(subset)
            subset = pitchtools.PitchClassSet(
                subset,
                item_class=pitchtools.NumberedPitchClass,
                )
            yield subset



# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class ChordOmission(AbjadObject):
    '''A chord ommission.

    Value object that can not be chnaged after instantiation.
    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self):
        pass



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Measure___delitem___01():
    r'''Nonnegative indices work.

    Automatically update time signature.
    '''

    measure = Measure((4, 8), "c'8 c'8 c'8 c'8")
    measure.automatically_adjust_time_signature = True
    del(measure[:1])

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 3/8
            c'8
            c'8
            c'8
        }
        '''
        )

    assert inspect_(measure).is_well_formed()


def test_scoretools_Measure___delitem___02():
    r'''Negative indices work.

    Automatically update time signatures.
    '''

    measure = Measure((4, 8), "c'8 c'8 c'8 c'8")
    measure.automatically_adjust_time_signature = True
    del(measure[-1:])

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 3/8
            c'8
            c'8
            c'8
        }
        '''
        )

    assert inspect_(measure).is_well_formed()


def test_scoretools_Measure___delitem___03():
    r'''Denominator preservation in time signature.

    Automatically update time signature.
    '''

    measure = Measure((4, 8), "c'8 c'8 c'8 c'8")
    measure.automatically_adjust_time_signature = True
    del(measure[:2])

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 2/8
            c'8
            c'8
        }
        '''
        )

    assert inspect_(measure).is_well_formed()


def test_scoretools_Measure___delitem___04():
    r'''Denominator changes from 8 to 16.

    Automatically update time signature.
    '''

    measure = Measure((4, 8), "c'16 c'16 c'8 c'8 c'8")
    measure.automatically_adjust_time_signature = True
    del(measure[:1])

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 7/16
            c'16
            c'8
            c'8
            c'8
        }
        '''
        )

    assert inspect_(measure).is_well_formed()


def test_scoretools_Measure___delitem___05():
    r'''Trim non-power-of-two measure.

    Automatically update time signature.
    '''

    measure = Measure((4, 9), "c'8 d'8 e'8 f'8")
    measure.implicit_scaling = True
    measure.automatically_adjust_time_signature = True
    del(measure[:1])

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 3/9
            \scaleDurations #'(8 . 9) {
                d'8
                e'8
                f'8
            }
        }
        '''
        )

    assert inspect_(measure).is_well_formed()


def test_scoretools_Measure___delitem___06():
    r'''Trim non-power-of-two measure, with denominator change.

    Automatically update time signature.
    '''

    measure = Measure((3, 9), "c'16 d'16 e'8 f'8")
    measure.implicit_scaling = True
    measure.automatically_adjust_time_signature = True

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 3/9
            \scaleDurations #'(8 . 9) {
                c'16
                d'16
                e'8
                f'8
            }
        }
        '''
        )

    del(measure[:1])

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 5/18
            \scaleDurations #'(8 . 9) {
                d'16
                e'8
                f'8
            }
        }
        '''
        )

    assert inspect_(measure).is_well_formed()


def test_scoretools_Measure___delitem___07():
    r'''Nonnegative indices work.

    Do NOT automatically update time signature.
    '''

    measure = Measure((4, 8), "c'8 c' c' c'")
    del(measure[:1])

    assert len(measure) == 3
    assert inspect_(measure).get_indicator(TimeSignature)
    assert not inspect_(measure).is_well_formed()


def test_scoretools_Measure___delitem___08():
    r'''Non-power-of-two measure.

    Do NOT automatically update time signature.
    '''

    measure = Measure((4, 9), "c'8 d' e' f'")
    measure.implicit_scaling = True
    del(measure[:1])

    assert len(measure) == 3
    assert inspect_(measure).get_indicator(TimeSignature)
    assert not inspect_(measure).is_well_formed()



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class LilyPondNameManager(AbjadObject):
    r'''LilyPond name manager.
    
    Base class from which grob, context setting and tweak managers inherit.
    '''

    ### SPECIAL METHODS ###

    def __eq__(self, arg):
        r'''Is true when `arg` is a LilyPond name manager with attribute pairs
        equal to those of this LilyPond name manager. Otherwise false.

        Returns true or false.
        '''
        if isinstance(arg, type(self)):
            return self._get_attribute_pairs() == arg._get_attribute_pairs()
        return False

    def __getstate__(self):
        r'''Gets object state.
        '''
        import copy
        return copy.deepcopy(vars(self))

    def __hash__(self):
        r'''Hashes LilyPond name manager.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(LilyPondNameManager, self).__hash__()

    def __repr__(self):
        r'''Gets interpreter representation of LilyPond name manager.

        Returns string.
        '''
        body_string = ''
        pairs = self._get_attribute_pairs()
        pairs = [str(_) for _ in pairs]
        body_string = ''.join(pairs)
        return '{}({})'.format(type(self).__name__, body_string)

    def __setstate__(self, state):
        r'''Sets object state.
        '''
        for key, value in state.items():
            self.__dict__[key] = value

    ### PRIVATE METHODS ###

    def _get_attribute_pairs(self):
        return tuple(sorted(vars(self).items()))



# -*- coding: utf-8 -*-
import re
import six


lowercamelcase_regex = re.compile(
    '^([a-z,0-9]+([A-Z,0-9]+[a-z,0-9]*)*)?$',
    re.VERBOSE,
    )


def is_lower_camel_case(argument):
    r'''Is true when `argument` is a string and is lowercamelcase.

    ..  container:: example

        ::

            >>> stringtools.is_lower_camel_case('fooBar')
            True

    Otherwise false:

    ..  container:: example

        ::

            >>> stringtools.is_lower_camel_case('FooBar')
            False

    Returns true or false.
    '''
    if not isinstance(argument, six.string_types):
        return False
    return bool(lowercamelcase_regex.match(argument))



# -*- coding: utf-8 -*-
import collections
from abjad.tools.datastructuretools.TypedCollection import TypedCollection
from abjad.tools.topleveltools import new


class TypedTuple(TypedCollection):
    r'''Typed tuple.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(
        self,
        items=None,
        item_class=None,
        ):
        TypedCollection.__init__(
            self,
            item_class=item_class,
            items=items,
            )
        items = items or []
        self._collection = tuple(
            self._item_coercer(item) for item in items)

    ### SPECIAL METHODS ###

    def __add__(self, argument):
        r'''Adds typed tuple to `argument`.

        Returns new typed tuple.
        '''
        if isinstance(argument, type(self)):
            items = argument._collection
            return new(self, items=self._collection[:] + items)
        elif isinstance(argument, type(self._collection)):
            items = argument[:]
            return new(self, items=self._collection[:] + items)
        raise NotImplementedError

    def __contains__(self, item):
        r'''Is true if typed tuple contains `item`.

        Coerces `item`.

        Returns none.
        '''
        try:
            item = self._item_coercer(item)
        except ValueError:
            return False
        return self._collection.__contains__(item)

    def __getitem__(self, argument):
        '''Gets item or slice identified by `argument`.

        Returns item or new typed tuple.
        '''
        item = self._collection.__getitem__(argument)
        try:
            return type(self)(item)
        except TypeError:
            return item

    def __hash__(self):
        r'''Hashes typed tuple.

        Returns integer.
        '''
        return hash((
            type(self),
            self._collection,
            self.item_class,
            ))

    def __mul__(self, argument):
        r'''Multiplies typed tuple by `argument`.

        Returns new typed tuple.
        '''
        items = self._collection * argument
        return new(self, items=items)

    def __radd__(self, argument):
        r'''Right-adds `argument` to typed tuple.
        '''
        items = argument + self._collection
        return new(self, items=items)

    def __rmul__(self, argument):
        r'''Multiplies `argument` by typed tuple.

        Returns new typed tuple.
        '''
        return self.__mul__(argument)

    ### PUBLIC METHODS ###

    def count(self, item):
        r'''Counts `item` in collection.

        Coerces `item`.

        Returns nonnegative integer.
        '''
        try:
            item = self._item_coercer(item)
        except TypeError:
            return 0
        return self._collection.count(item)

    def index(self, item):
        r'''Gets index of `item` in collection.

        Coerces `item`.

        Returns nonnegative integer.
        '''
        item = self._item_coercer(item)
        return self._collection.index(item)


collections.Sequence.register(TypedTuple)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Measure__get_likely_multiplier_of_components_01():
    r'''Components were likely multiplied by 5/4.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    mutate(staff).scale(Multiplier(5, 4))
    assert Measure._get_likely_multiplier_of_components(staff[:]) == \
        Multiplier(5, 4)


def test_scoretools_Measure__get_likely_multiplier_of_components_02():
    r'''Components were likely multiplied by 3/2.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    mutate(staff).scale(Multiplier(3, 2))
    assert Measure._get_likely_multiplier_of_components(staff[:]) == \
        Multiplier(3, 2)


def test_scoretools_Measure__get_likely_multiplier_of_components_03():
    r'''Components were likely multiplied by 7/4.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    mutate(staff).scale(Multiplier(7, 4))
    assert Measure._get_likely_multiplier_of_components(staff[:]) == \
        Multiplier(7, 4)


def test_scoretools_Measure__get_likely_multiplier_of_components_04():
    r'''Components likely multiplier not recoverable.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    mutate(staff).scale(Multiplier(2))
    assert Measure._get_likely_multiplier_of_components(staff[:]) == \
        Multiplier(1)


def test_scoretools_Measure__get_likely_multiplier_of_components_05():
    r'''Components likely multiplier not recoverable.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    mutate(staff).scale(Multiplier(1, 2))
    assert Measure._get_likely_multiplier_of_components(staff[:]) == \
        Multiplier(1)


def test_scoretools_Measure__get_likely_multiplier_of_components_06():
    r'''Components multiplier recoverable only to within one power of two.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    mutate(staff).scale(Multiplier(10, 4))
    assert not Measure._get_likely_multiplier_of_components(staff[:]) == \
        Multiplier(10, 4)
    assert Measure._get_likely_multiplier_of_components(staff[:]) == \
        Multiplier(5, 4)


def test_scoretools_Measure__get_likely_multiplier_of_components_07():
    r'''Returns none when more than one likely multiplier.
    '''

    staff = Staff(scoretools.make_notes([0], [(1, 8), (7, 32)]))
    assert Measure._get_likely_multiplier_of_components(staff[:]) is None



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSegment___rmul___01():

    pitch_class_segment_1 = pitchtools.PitchClassSegment([0, 1, 11, 9])
    pitch_class_segment_2 = 2 * pitch_class_segment_1

    assert pitch_class_segment_2 == \
        pitchtools.PitchClassSegment([0, 1, 11, 9, 0, 1, 11, 9])



# -*- coding: utf-8 -*-


def example_function(argument):
    r'''This is a multiline docstring.

    This is the third line of the docstring.
    '''
    # This is a comment.
    print('Entering example function.')
    try:
        argument = argument + 1
    except TypeError:
        print('Wrong type!')
    print(argument)
    print('Leaving example function.')
